#!/usr/bin/perl

# (c) vbabin-at-nowhere-kz (2010)

use strict;
use warnings;

my $T = 300.0;
my $kB = 8.31441E-3/4.184;
my $kBT = $T*$kB;

die " ** Error ** : need 2 file names\n" unless defined(@ARGV)
  and defined($ARGV[0]) and defined($ARGV[1]);

my @WF = ();
&read_work($ARGV[0], \@WF);
my $nF = scalar @WF;
print "<> A -> B (forward) $nF work values loaded from '$ARGV[0]'\n";

my @WR = ();
&read_work($ARGV[1], \@WR);
my $nR = scalar @WR;
print "<> B -> A (reverse) $nR work values loaded from '$ARGV[1]'\n";

die " ** Error ** : nothing to do\n" if ($nF == 0 or $nR == 0);

#
# find upper/lower bounds
#

my $W_hi = + &mean(@WF);
my $W_lo = - &mean(@WR);

print "<> delta-f bounds (work means): $W_hi / $W_lo\n";
print "<> Jarzynski [forward]: ", + &jarzynski(@WF), "\n";
print "<> Jarzynski [reverse]: ", - &jarzynski(@WR), "\n";
print ">>> >>> Bennet (fwd/rev) : ", &bennet(\@WF, \@WR), "\n";

sub read_work()
{
    my $filename = shift;
    my $aref = shift;

    open FH, "$filename" or
       die " ** Error ** : could not open '$filename' for reading: $!\n";

    while (<FH>) {
        next if (/^\#/);
        chomp;
        my @toks = split;
        die " ** Error ** : unexpected text '$_' in '$filename'\n"
            if scalar @toks > 1;

        push @{$aref}, $toks[0] if scalar @toks == 1;
    }

    close FH;
}

sub mean {
    my $accu = 0.0;
    foreach (@_) {
        $accu += $_;
    }
    return $accu/(scalar @_);
}

sub jarzynski {
    my $lo = $_[0];
    foreach (@_) {
        $lo = $_ unless $lo < $_;
    }

    my $accu = 0.0;
    foreach (@_) {
        my $delta = $lo - $_;
        $accu += exp($delta/$kBT);
    }

    $accu /= scalar @_;

    return $lo - $kBT*log($accu);
}

sub bennet {
    my $WFr = shift;
    my $WRr = shift;

    my ($xl, $xr) = (- &mean(@{$WRr}), + &mean(@{$WFr}));
    my ($rl, $rr) = (&residue($WFr, $WRr, $xl), &residue($WFr, $WRr, $xr));

    while ($xr - $xl > 0.00001) {
        my $xc = ($xr + $xl)/2.0;
        my $rc = &residue($WFr, $WRr, $xc);

        if ($rc*$rl <= 0.0) {
            $rr = $rc;
            $xr = $xc;
        } elsif ($rc*$rr <= 0.0) {
            $rl = $rc;
            $xl = $xc;
        } else {
            die "*** science over ***\n";
        }
    }

    return ($xl + $xr)/2.0;
}

sub residue
{
    my $WFr = shift;
    my $WRr = shift;

    my ($nF, $nR) = (scalar @{$WFr}, scalar @{$WRr});
    die unless $nF > 0 and $nR > 0;

    my $x = shift;

    my $accu = 0.0;

    for (my $k = 0; $k < $nF; ++$k) {
        my $delta = $WFr->[$k] - $x;
        $accu += 1.0/(1.0 + ($nF/$nR)*exp($delta/$kBT));
    }

    for (my $k = 0; $k < $nR; ++$k) {
        my $delta = $x + $WRr->[$k];
        $accu -= 1.0/(1.0 + ($nR/$nF)*exp($delta/$kBT));
    }

    return $accu;
}
