#!/usr/bin/perl

# (c) vbabin-at-ncsu-dot-edu 2010

use strict;
use warnings;

my $kB = 8.31441E-3/4.184;
my $kBT = 300.0*$kB;

die "usage: omegas.pl o1 [o2 ...]\n"
    unless defined(@ARGV) and scalar @ARGV >= 1;

my @trans = ();

foreach (@ARGV) {
    &do_one($_, \@trans);
}

my ($all_cis, $all_trans) = (1.0, 1.0);
print "\ntrans populations:\n";
foreach (@trans) {
   printf " %.4f", $_;
   $all_trans *= $_;
   $all_cis *= (1.0 - $_);
}
printf "\n\nall-trans probability: %.10f\n", $all_trans;
printf "all-cis   probability: %.10f\n", $all_cis;

print "==> free energy difference: ",
    $kBT*log($all_trans/$all_cis), " (kcal/mol)\n";

my %pigs;
my $n_omega = scalar @trans;

for (my $n = 0; $n < (1 << $n_omega); ++$n) {
    my $prob = 1.0;
    my $seq = '';
    for (my $k = 0; $k < $n_omega; ++$k) {
        my $is_trans = ($n & (1 << $k)) >> $k;
        $seq .= ($is_trans ? 'T' : 'C');
        $prob *= ($is_trans ? $trans[$k] : (1.0 - $trans[$k]));
    }
    $pigs{$prob} = $seq;
}

print "\nprobabilities:\n";
my @sortedkeys = sort {$a <=> $b} keys(%pigs);
foreach (reverse(@sortedkeys)) {
    printf "$pigs{$_} : %.10f\n", $_;
}

sub do_one {
    my $filename = shift;

    open FH, "$filename"
        or die "! could not open '$filename' for reading\n";

    my ($n_cis, $n_trans) = (0, 0);

    while (<FH>) {
        my @toks = split;
        my $angle = $toks[1];
        if ($angle > -90.0 and $angle < 90.0) {
            ++$n_cis;
        } else {
            ++$n_trans;
        }
    }

    close FH;

    my $n_total = $n_cis + $n_trans;
    my $ar = shift;

    push(@{$ar}, $n_trans/$n_total);
    printf "'$filename' : #cis = %6d, #trans = %6d\n", $n_cis, $n_trans;
}
